from sqlalchemy.orm import Mapped
from typing import Any, Self

import requests


from portal.model.Base import Base


class TaigaUserStoryProvider(Base):
    """
    Fetches a list of "user stories" (i.e., tasks) from the project management tool Taiga.
    """

    url: Mapped[str]
    authorization: Mapped[str]

    def fetch_user_stories(self) -> list[dict[str, Any]]:
        return requests.get(
            self.url, headers={"Authorization": self.authorization}
        ).json()

    @classmethod
    def get_the_one(cls) -> Self:
        instances = cls.get_all()
        if len(instances) != 1:
            raise ValueError(
                f"Expected there to be exactly 1 instance of {cls.__name__}, but there were {len(instances)}"
            )

        return instances[0]

    def to_dict(self) -> dict[str, Any]:
        return {"uuid": self.uuid, "url": self.url}
